@@ -2,6 +2,7 @@ import pandas as pd
 import numpy as np
 import wash_tool
 import data_fenxi
+from parseIdCard import parseIdCard
 from datetime import datetime
 
 
@@ -34,8 +35,8 @@ class BankData:
             must_line = ['主端卡号','主端账号','主端姓名','交易时间','交易金额','交易余额','收付标志','对端卡号','对端姓名','交易结果','交易摘要','交易网点']
             extra_line = ['对端银行','IP地址','MAC地址','商户名称','交易类型','备注']
             
-            rename_dicts = {'主端卡号': ['交易卡号','查询卡号'],
-                            '主端账号': ['交易账号','查询账号'],
+            rename_dicts = {'主端卡号': ['交易卡号','查询卡号','查询账号'],
+                            '主端账号': ['交易账号'],
                             '主端姓名': ['交易方户名','交易户名'],
                             '交易时间': ['交易时间'],
                             '收付标志': ['借贷标志'],
@@ -63,7 +64,10 @@ class BankData:
     
     def wash_card_deal(self):
         new_df = self.detail.copy()
-        new_df = new_df.applymap(lambda x:x if pd.isnull(x) else str(x).replace('', '')) #去除非法字符
+        # new_df = new_df.applymap(lambda x:x if pd.isnull(x) else str(x).replace('', '')) #去除非法字符
+        illegal_chars = wash_tool.detect_illegal_characters(new_df)
+        new_df = wash_tool.clean_illegal_characters(new_df, illegal_chars)
+        
         new_df['收付标志'].replace(['C','贷','入'],'进', inplace=True)
         new_df['收付标志'].replace(['D','借'],'出', inplace=True)
         new_df = new_df[new_df['收付标志'].isin(['进','出'])]
@@ -102,7 +106,10 @@ class BankData:
         new_df.drop_duplicates(subset=['交易标识'],keep='first',inplace=True)
         
         bad_lines = self.detail[~self.detail.index.isin(new_df.index.tolist())]
-        bad_lines = bad_lines.applymap(lambda x:x if pd.isnull(x) else str(x).replace('', ''))
+        # bad_lines = bad_lines.applymap(lambda x:x if pd.isnull(x) else str(x).replace('', ''))
+        
+        illegal_chars_b = wash_tool.detect_illegal_characters(bad_lines)
+        bad_lines = wash_tool.clean_illegal_characters(bad_lines, illegal_chars_b)
         
         return new_df.reset_index(drop=True),bad_lines.reset_index(drop=True)
     
@@ -137,7 +144,7 @@ class JZBankData(BankData):
             self.detail = self.detail[self.detail['主端卡号'].isin(self.diao['调取卡号'])]
             print('清洗后数据:',len(self.detail),'行','  清洗掉数据:',len(self.bad_line),'行','  反馈外数据:',len(self.useless),'行')
         else:
-            print('清洗后数据:',len(self.detail),'行','  清洗掉数据:',len(self.bad_line))
+            print('清洗后数据:',len(self.detail),'行','  清洗掉数据:',len(self.bad_line),'行')
     
     @staticmethod  
     def adjust_card(x):
@@ -146,8 +153,37 @@ class JZBankData(BankData):
                 return x['交易卡号']
         return x['主端卡号']
     
+    @staticmethod
+    def people_unification_name_and_get_line(df_rename):
+            must_line = ['客户名称','证照类型','证照号码','住址地址','单位地址','联系电话','联系手机','单位电话','住宅电话','工作单位','邮箱地址']
+            extra_line = ['代办人姓名','代办人证件类型','代办人证件号码']
+            
+            all_line = must_line + extra_line
+            for line in all_line:
+                if line not in df_rename.columns:
+                    df_rename[line] = ''
+                
+            df_rename = df_rename[all_line]
+            return df_rename
+        
+        
+    @staticmethod    
+    def card_unification_name_and_get_line(df_rename):
+        must_line = ['交易卡号','账户开户名称','开户人证件号码','交易账号','账号开户时间','账户余额','可用余额','币种','开户网点代码','开户网点','账户状态','炒汇标志名称','销户日期','账户类型','开户联系方式','通信地址','联系电话','备注','账号开户银行','销户网点','最后交易时间',]
+        extra_line = ['来源']
+        
+        all_line = must_line + extra_line
+        for line in all_line:
+            if line not in df_rename.columns:
+                df_rename[line] = ''
+            
+        df_rename = df_rename[all_line]
+        return df_rename
+    
+    
     def others_msg_clean(self):
         zh = self.qin_xi(self.user,['交易账号','交易卡号'])
+        zh = self.card_unification_name_and_get_line(zh)
         zh = zh.rename(columns={'交易账号':'主端账号'})
         zh.insert(0, '交易卡号', zh.pop('交易卡号'))
         
@@ -158,10 +194,13 @@ class JZBankData(BankData):
         zh_name =zh.query('主端账号 != "" and 账户开户名称 != ""')
         self.name_z = zh[['主端账号','账户开户名称']].drop_duplicates(keep='first')
         
-        zh = zh.query('主端账号 != "" and 交易卡号 != ""')
+        zh = zh.query('主端账号 != "" and 交易卡号 != "" and 交易卡号 != "0"')
         self.user_all = zh.dropna(subset=['主端账号', '交易卡号'], how='all')
-        self.user_all = self.user_all.applymap(lambda x:x if pd.isnull(x) else str(x).replace('', ''))
-
+        
+        # self.user_all = self.user_all.applymap(lambda x:x if pd.isnull(x) else str(x).replace('', ''))
+        
+        illegal_chars_z = wash_tool.detect_illegal_characters(self.user_all)
+        self.user_all = wash_tool.clean_illegal_characters(self.user_all, illegal_chars_z)
         
         tmp_zh = zh[['主端账号','交易卡号']].drop_duplicates(keep='first')
         value_counts = tmp_zh['主端账号'].value_counts()   # 选取一对一的账户匹配避免重复数据
@@ -172,10 +211,12 @@ class JZBankData(BankData):
 
         #人员信息
         pe = self.qin_xi(self.people)
-        # pe['户籍地'] = pe['证照号码'].apply(parseIdCard.parseIdCard).apply(lambda x : x['area'])
-        # pe['年龄'] = pe['证照号码'].apply(parseIdCard.parseIdCard).apply(lambda x : x['age'])
-        # pe.insert(3, '户籍地', pe.pop('户籍地'))
-        # pe.insert(4, '年龄', pe.pop('年龄'))
+        pe = self.people_unification_name_and_get_line(pe)
+        pe['户籍地'] = pe['证照号码'].apply(parseIdCard.parseIdCard).apply(lambda x : x['area'] if 'area' in x else '')
+        pe['年龄'] = pe['证照号码'].apply(parseIdCard.parseIdCard).apply(lambda x : x['age'] if 'age' in x else '')
+        pe.insert(3, '户籍地', pe.pop('户籍地'))
+        pe.insert(4, '年龄', pe.pop('年龄'))
+        pe = pe.query('客户名称 != ""')
         self.people = pe
         
         # 冻结信息清洗
@@ -185,8 +226,12 @@ class JZBankData(BankData):
         dj = pd.merge(dj,self.user,left_on='主端卡号',right_on='主端账号',how='left')
         dj['主端卡号'] = dj.apply(lambda x: x['交易卡号'] if pd.notnull(x['交易卡号']) else x['主端卡号'],axis=1)
         dj.drop(['主端账号','交易卡号'],axis=1,inplace=True)
-        self.freeze = data_fenxi.location_extract(dj,'冻结机关') # 标准化公安地址
-        self.freeze = self.freeze.applymap(lambda x:x if pd.isnull(x) else str(x).replace('', ''))
+        self.freeze = wash_tool.location_extract(dj,'冻结机关') # 标准化公安地址
+        # self.freeze = self.freeze.applymap(lambda x:x if pd.isnull(x) else str(x).replace('', ''))
+
+        illegal_chars_f = wash_tool.detect_illegal_characters(self.freeze)
+        self.freeze = wash_tool.clean_illegal_characters(self.freeze, illegal_chars_f)
+
 
         #任务反馈清洗
         if not self.task.empty:
@@ -221,6 +266,10 @@ class JZBankData(BankData):
             new_df['主端姓名'] = new_df['主端姓名'].fillna(new_df['姓名'])
             new_df.drop(['姓名'], axis=1,inplace=True)
         
+        new_df['主端姓名'] = new_df['主端姓名'].str.replace(' 先生', '', regex=False)
+        new_df['主端姓名'] = new_df['主端姓名'].str.replace(' 女士', '', regex=False)
+     
+        
         # 新增列
         # 对端赌客匹配
         if not self.duke_card.empty:
@@ -264,14 +313,14 @@ class JZBankData(BankData):
         all_presently_money = int(presently_money['交易余额'].sum())
         money_water = int(alldata['交易金额'].sum())
         
-        fankui_table = pd.merge(data_sum,dj_card,how='outer',on='主端卡号') # 添加数据量信息
+        fankui_table = pd.merge(data_sum,dj_card,how='left',on='主端卡号') # 添加数据量信息
         fankui_table = pd.merge(fankui_table,presently_money.reset_index()[['主端卡号','交易时间','交易余额']],how='outer',on='主端卡号')
         fankui_table.rename(columns={'交易时间':'最后交易时间'},inplace=True)
         
         if not self.diao.empty:
-            fankui_table = pd.merge(self.diao,fankui_table,how='outer',left_on='调取卡号',right_on='主端卡号')
+            fankui_table = pd.merge(self.diao[['调取卡号','姓名','归属行','批次']],fankui_table,how='left',left_on='调取卡号',right_on='主端卡号')
             
-        freeze_presently_money = fankui_table[fankui_table['冻结次数'].isnull()]['交易余额'].sum()
+        freeze_presently_money = fankui_table[fankui_table['冻结次数'].notnull()]['交易余额'].sum()
             
         dongjie_card = fankui_table['冻结次数'].count()
         fankuishui = fankui_table['主端卡号'].count()
@@ -344,6 +393,10 @@ class ZABankData(BankData):
             new_df = pd.merge(new_df,d_name, on='主端卡号',how='left')
             new_df['主端姓名'] = new_df['主端姓名'].fillna(new_df['姓名'])
             new_df.drop(['姓名'], axis=1,inplace=True)
+            
+            new_df['主端姓名'] = new_df['主端姓名'].str.replace(' 先生', '', regex=False)
+            new_df['主端姓名'] = new_df['主端姓名'].str.replace(' 女士', '', regex=False)
+        
         if not self.duke_card.empty:
             dk = self.duke_card[['赌客卡号']].drop_duplicates(keep='first')
             new_df['对端isDuke'] = new_df['对端卡号'].isin(dk['赌客卡号'])
@@ -351,11 +404,14 @@ class ZABankData(BankData):
 
     def report(self):
         alldata = self.detail.copy()
-
+        
         data_sum = (pd.pivot_table(alldata[['主端卡号','交易金额']],index=["主端卡号"],values='交易金额',aggfunc=['sum','count']).reset_index())
         data_sum.columns = ['_'.join(col) for col in data_sum.columns.values]
         data_sum.rename(columns={'sum_交易金额':'交易流水','count_交易金额':'数据量','主端卡号_':'主端卡号'}, inplace=True)
         data_sum = data_sum[data_sum['主端卡号'] != '']
+        data_sum = pd.merge(data_sum,alldata[['主端卡号','主端姓名']].drop_duplicates(keep='first'),on='主端卡号',how='left')
+        data_sum.insert(1, '主端姓名', data_sum.pop('主端姓名'))
+        data_sum['冻结次数'] = ''
         
         rows = ' '
         if not self.diao.empty:
@@ -373,7 +429,7 @@ class ZABankData(BankData):
         fankui_table.rename(columns={'交易时间':'最后交易时间'},inplace=True)
         
         if not self.diao.empty:
-            fankui_table = pd.merge(self.diao,fankui_table,how='outer',left_on='调取卡号',right_on='主端卡号')
+            fankui_table = pd.merge(self.diao[['调取卡号','姓名','归属行','批次']],fankui_table,how='left',left_on='调取卡号',right_on='主端卡号')
             
         fankuishui = fankui_table['主端卡号'].count()
         
